# Copyright (c) 2001-2002 Alexander Kanavin. All rights reserved.

""" This module contains GUI class that deals with file transfers
"""

from wxPython.wx import *
from pysoulseek import slskmessages
import os
import pysoulseek
import string
import time
from sortablelist import sortableListCtrl

class TransfersList(sortableListCtrl):
    """ This is a list control for transfers. Gets transfer data from transfer
    manager.
    """
    def __init__(self, parent, id, list, style = wxLC_REPORT|wxLC_VIRTUAL|wxSUNKEN_BORDER):
        sortableListCtrl.__init__(self,parent,id, style = style)
	self.list = list
	self.InsertColumn(0,"Filename", width = 250)
	self.InsertColumn(1,"User", width = 100)
	self.InsertColumn(2,"Status", width  = 150)
	self.InsertColumn(3,"Size", width = 100)
	self.InsertColumn(4,"Speed",width = 50)
	self.InsertColumn(5,"Time elapsed", width = 70)
	self.InsertColumn(6,"Time left", width = 70)
	self.InsertColumn(7,"Path", width = 100)
	self.SetItemCount(len(list))


    def OnGetItemText(self, item,col):
	if len(self.list) <= item:
	    return None
	else:
	    return self.GetColumnText(self.list[item],col)

    def GetColumnText(self,item,col):
	if col == 0:
	    return string.split(item.filename,'\\')[-1]
	if col == 1:
	    return item.user
	if col == 2:
	    return item.status
	if col == 3:
	    return item.size
	if col == 4:
            if item.speed is not None:
	        return "%.1f" %(item.speed)
	if col == 5:
	    return item.timeelapsed
	if col == 6:
	    return item.timeleft
	if col == 7:
	    return item.path

    def OnGetItemImage(self,item):
	return -1

    def UnselectAll(self):
        for i in range(len(self.list)):
            self.SetItemState(i,0,wxLIST_STATE_SELECTED)

    def SortList(self, col, order):
        if order == 0:
            self.list.sort(lambda x,y: cmp(self.GetColumnText(x,col),self.GetColumnText(y,col)))
        else:
            self.list.sort(lambda y,x: cmp(self.GetColumnText(x,col),self.GetColumnText(y,col)))


class TransfersPanel(wxPanel):
    """ This is the transfer panel that contains transfers list control.
    The right-click pop--up menu is implemented here."""
    def __init__(self, parent, id, list, eventprocessor, transfermanager):
	wxPanel.__init__(self,parent,id)
	self.list = list
	self.eventprocessor = eventprocessor
	self.transfermanager = transfermanager

	self.processrequest = self.eventprocessor.ProcessRequestToPeer
	self.users = self.transfermanager.users
	self.listctrl = TransfersList(self,-1,list)
	sizer = wxBoxSizer(wxVERTICAL)
	sizer.Add(self.listctrl,1,wxEXPAND, border = 10)
	self.SetSizer(sizer)
	self.SetAutoLayout(true)

	self.menu = wxMenu()
	getplaceID = wxNewId()
	self.menu.Append(getplaceID, 'Get Place In Queue')
	EVT_MENU(self,getplaceID,self.OnGetPlace)
        sendmessageID=wxNewId()
        self.menu.Append(sendmessageID, 'Send Message')
        EVT_MENU(self,sendmessageID, self.OnSendMessage)
        getinfoID=wxNewId()
        self.menu.Append(getinfoID, 'Get User Info')
        EVT_MENU(self,getinfoID, self.OnGetInfo)
        browseID=wxNewId()
        self.menu.Append(browseID, 'Browse Files')
        EVT_MENU(self,browseID, self.OnBrowse)
        addtolistID=wxNewId()
        self.menu.Append(addtolistID, 'Add to User List')
        EVT_MENU(self,addtolistID, self.OnAddToList)

	self.menu.AppendSeparator()

        abortID=wxNewId()
        self.menu.Append(abortID, 'Abort')
        EVT_MENU(self,abortID, self.OnAbort)
        retryID=wxNewId()
        self.menu.Append(retryID, 'Retry')
        EVT_MENU(self,retryID, self.OnRetry)
        clearID=wxNewId()
        self.menu.Append(clearID, 'Clear')
        EVT_MENU(self,clearID, self.OnClear)
        self.menu.AppendSeparator()

        clearfaID=wxNewId()
        self.menu.Append(clearfaID, 'Clear finished/aborted')
        EVT_MENU(self,clearfaID, self.OnClearFinAb)
        clearfID=wxNewId()
        self.menu.Append(clearfID, 'Clear finished')
        EVT_MENU(self,clearfID, self.OnClearFin)
        clearaID=wxNewId()
        self.menu.Append(clearaID, 'Clear aborted')
        EVT_MENU(self,clearaID, self.OnClearAb)
        clearqID=wxNewId()
        self.menu.Append(clearqID, 'Clear queued')
        EVT_MENU(self,clearqID, self.OnClearQueued)


	EVT_RIGHT_UP(self.listctrl, self.OnRightUp)

    def update(self):
	""" Call this to make the transfer panel update the data."""
	self.listctrl.SetItemCount(len(self.list))

    def OnRightUp(self, event):
	""" Show the pop-up menu"""
        pt = event.GetPosition()
        item, flags = self.listctrl.HitTest(pt)
	self.item = item
	if item >= 0:
            self.listctrl.SetItemState(item,wxLIST_STATE_FOCUSED,wxLIST_STATE_FOCUSED)
            self.listctrl.PopupMenu(self.menu, pt)

    """ Handlers for the menu items """

    def OnSendMessage(self, event):
	""" Sends user a private message """
        self.eventprocessor.privatechat.SendMessage(self.list[self.item].user)

    def OnGetPlace(self,event):
	""" Gets place in line for the queued file """
        user = self.list[self.item].user
        if user in self.users.keys() and self.users[user].status > 0:
            self.processrequest(user, slskmessages.PlaceInQueueRequest(None,self.list[self.item].filename))

    def OnGetInfo(self, event):
	""" Gets info about a user """
	user = self.list[self.item].user
        if user in self.users.keys() and self.users[user].status > 0:
            self.processrequest(user, slskmessages.UserInfoRequest(None), self.eventprocessor.userinfo)

    def OnBrowse(self, event):
	""" Browse user's files """
	user = self.list[self.item].user
        if user in self.users.keys() and self.users[user].status > 0:
            self.processrequest(user, slskmessages.GetSharedFileList(None), self.eventprocessor.userbrowse)

    def OnAddToList(self,event):
	user = self.list[self.item].user
	self.eventprocessor.frame.AddToList(user)	

    def OnAbort(self, event):
	""" Abort all selected transfers """
	item = -1
	while 1:
	    item = self.listctrl.GetNextItem(item,wxLIST_NEXT_ALL,wxLIST_STATE_SELECTED|wxLIST_STATE_FOCUSED)
	    if item == -1:
		break

	    transfer = self.list[item]
            if transfer.status != "Finished" and transfer.status != "Old":
		self.transfermanager.AbortTransfer(transfer)
                transfer.status = "Aborted"
		transfer.req = None
		transfer.remotereq = None
	if self.list == self.transfermanager.uploads:
	    self.transfermanager.checkUploadQueue()
	self.update()

    def OnRetry(self,event):
	""" Retry all selected transfers, if they're not active 
	and not finished and a user is online """
        item = -1
        while 1:
            item = self.listctrl.GetNextItem(item,wxLIST_NEXT_ALL,wxLIST_STATE_SELECTED|wxLIST_STATE_FOCUSED)
            if item == -1:
                break
            transfer = self.list[item]
            if transfer.status != "Finished" and transfer.status != "Old":
                self.transfermanager.AbortTransfer(transfer)
                transfer.req = None
                transfer.remotereq = None
	        if self.list == self.transfermanager.downloads:
		    self.transfermanager.getFile(transfer.user, transfer.filename, transfer.path, transfer)

    def OnClear(self,event):
	""" Remove all selected transfers. Abort those that are in progress."""
        item = -1
        while 1:
            item = self.listctrl.GetNextItem(item,wxLIST_NEXT_ALL,wxLIST_STATE_SELECTED|wxLIST_STATE_FOCUSED)
            if item == -1:
                break
            transfer = self.list[item]
            self.transfermanager.AbortTransfer(transfer)
	    self.list[item] = None
	while self.list.count(None) > 0:
	    self.list.remove(None)
	self.listctrl.UnselectAll()
        if self.list == self.transfermanager.uploads:
            self.transfermanager.checkUploadQueue()
 	self.update()

    def OnClearFinAb(self,event):
	""" Remove all finished or aborted transfers"""
	for i in self.list[:]:
	    if i.status == "Finished" or i.status == "Aborted" or i.status == "Connection closed by peer" or i.status == "Cancelled":
		self.list.remove(i)
        self.listctrl.UnselectAll()
	self.update()

    def OnClearFin(self,event):
	""" Remove just the finished transfers."""
        for i in self.list[:]:
            if i.status == "Finished":
                self.list.remove(i)
        self.listctrl.UnselectAll()
        self.update()

    def OnClearAb(self,event):
	""" Remove just the aborted transfers. """
        for i in self.list[:]:
            if i.status == "Aborted" or i.status == "Connection closed by peer" or i.status== "Cancelled":
                self.list.remove(i)
        self.listctrl.UnselectAll()
        self.update()

    def OnClearQueued(self,event):
	""" Remove all (locally or remotely) queued transfers. """
        for i in self.list[:]:
            if i.status == "Queued":
                self.list.remove(i)
        self.listctrl.UnselectAll()
        self.update()


